import bpy

class AddPassiveRigidBody(bpy.types.Operator):
    """Add a passive rigid body to the selected object"""
    bl_idname = "object.add_passive_rigid_body"
    bl_label = "Add Passive Rigid Body"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        obj = context.active_object
        return obj and obj.type == 'MESH'

    def execute(self, context):
        obj = context.active_object
        if not obj or obj.type != 'MESH':
            self.report({'ERROR'}, "Rigid bodies can only be added to mesh objects")
            return {'CANCELLED'}
            
        bpy.ops.rigidbody.objects_add(type='PASSIVE')
        return {'FINISHED'}

class VIEW3D_MT_quick_rigid_collision_submenu(bpy.types.Menu):
    """Collision shape settings submenu"""
    bl_label = "Collision Shape"
    bl_idname = "VIEW3D_MT_quick_rigid_collision_submenu"

    def draw(self, context):
        layout = self.layout
        obj = context.active_object
        
        if obj and obj.rigid_body:
            layout.prop_enum(obj.rigid_body, "collision_shape", 'BOX', icon='MESH_CUBE')
            layout.prop_enum(obj.rigid_body, "collision_shape", 'SPHERE', icon='MESH_UVSPHERE')
            layout.prop_enum(obj.rigid_body, "collision_shape", 'CAPSULE', icon='MESH_CAPSULE')
            layout.prop_enum(obj.rigid_body, "collision_shape", 'CYLINDER', icon='MESH_CYLINDER')
            layout.prop_enum(obj.rigid_body, "collision_shape", 'CONE', icon='MESH_CONE')
            layout.prop_enum(obj.rigid_body, "collision_shape", 'CONVEX_HULL', icon='MOD_MESHDEFORM')
            layout.prop_enum(obj.rigid_body, "collision_shape", 'MESH', icon='MESH_MONKEY')
            
            layout.separator()
            layout.prop(obj.rigid_body, "use_margin", text="Use Collision Margin")
            if obj.rigid_body.use_margin:
                layout.prop(obj.rigid_body, "collision_margin", text="Margin")

class VIEW3D_MT_quick_rigid(bpy.types.Menu):
    """Quick access to rigid body tools"""
    bl_label = "Quick Rigid"
    bl_idname = "VIEW3D_MT_quick_rigid"

    def draw(self, context):
        layout = self.layout
        layout.operator_context = 'INVOKE_DEFAULT'
        
        obj = context.active_object
        has_rigidbody = obj and obj.rigid_body

        # Add/Remove section
        layout.operator("rigidbody.object_add", text="Add Active Rigid Body", icon='MESH_MONKEY')
        layout.operator("object.add_passive_rigid_body", text="Add Passive Rigid Body", icon='MESH_CUBE')
        
        if has_rigidbody:
            layout.separator()
            layout.operator("rigidbody.object_remove", text="Remove Rigid Body", icon='X')
            
            layout.separator()
            
            # Type indicator
            layout.label(text=f"Type: {obj.rigid_body.type.title()}", icon='PHYSICS')
            
            # Animation
            layout.prop(obj.rigid_body, "kinematic", text="Animated", icon='RENDER_ANIMATION', toggle=True)
            
            layout.separator()
            
            # Collision Shape submenu
            layout.menu("VIEW3D_MT_quick_rigid_collision_submenu", text="Collision Shape", icon='MESH_ICOSPHERE')
            
            # Mass settings
            layout.operator("rigidbody.mass_calculate", text="Calculate Mass", icon='PHYSICS')
            layout.prop(obj.rigid_body, "mass", text="Mass", slider=True)
            
            # Physics properties
            layout.prop(obj.rigid_body, "friction", text="Friction", slider=True)
            layout.prop(obj.rigid_body, "restitution", text="Bounciness", slider=True)
            
            # Gravity settings
            layout.separator()
            layout.prop(context.scene, "use_gravity", text="Use Gravity")
            if context.scene.use_gravity:
                col = layout.column(align=True)
                col.prop(context.scene, "gravity", text="X", index=0)
                col.prop(context.scene, "gravity", text="Y", index=1)
                col.prop(context.scene, "gravity", text="Z", index=2)
            
            # Simulation settings
            if context.scene.rigidbody_world:
                layout.prop(context.scene.rigidbody_world, "time_scale", text="Sim Speed", slider=True)
                layout.prop(context.scene.rigidbody_world, "solver_iterations", text="Solver Iterations")
            
            layout.separator()
            
            # Copy/Paste
            layout.operator("rigidbody.object_settings_copy", text="Copy Settings", icon='COPYDOWN')
            
            layout.separator()
            
            # Bake
            layout.operator("rigidbody.bake_to_keyframes", text="Bake to Keyframes", icon='KEY_HLT')

class VIEW3D_PT_QuickRigid(bpy.types.Panel):
    """Panel for Quick Rigid tools"""
    bl_label = "Quick Rigid"
    bl_idname = "VIEW3D_PT_quick_rigid"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "Quick Rigid"
   
    def draw(self, context):
        layout = self.layout
        layout.scale_y = 1.2
        
        if not context.selected_objects:
            layout.label(text="Select a mesh to add a rigid body", icon='INFO')
            return
            
        obj = context.active_object
        if not obj or obj.type != 'MESH' or not obj.select_get():
            layout.label(text="Select a mesh to add a rigid body", icon='INFO')
            return

        # Add rigid body buttons
        row = layout.row(align=True)
        row.operator("rigidbody.object_add", text="Add Active", icon='MESH_MONKEY')
        row.operator("object.add_passive_rigid_body", text="Add Passive", icon='MESH_CUBE')

        if not obj.rigid_body:
            return

        layout.separator()
        
        # Type indicator
        box = layout.box()
        row = box.row()
        icon = 'PINNED' if obj.rigid_body.type == 'PASSIVE' else 'UNPINNED'
        row.label(text=f"Type: {obj.rigid_body.type.title()}", icon=icon)
        
        # Remove button
        row = layout.row()
        row.operator("rigidbody.object_remove", text="Remove Rigid Body", icon='X')
        
        # Main settings
        box = layout.box()
        col = box.column(align=True)
        col.prop(obj.rigid_body, "collision_shape", text="Shape", icon='MESH_ICOSPHERE')
        col.prop(obj.rigid_body, "kinematic", text="Animated", icon='RENDER_ANIMATION', toggle=True)
        
        # Copy/Paste
        row = layout.row(align=True)
        row.operator("rigidbody.object_settings_copy", text="Copy Settings", icon='COPYDOWN')
        
        # Mass settings
        box = layout.box()
        box.label(text="Mass:", icon='PHYSICS')
        col = box.column(align=True)
        col.prop(obj.rigid_body, "mass", slider=True)
        col.operator("rigidbody.mass_calculate", text="Calculate Mass", icon='FILE_REFRESH')

        # Surface Response
        box = layout.box()
        box.label(text="Surface Response:", icon='MOD_PHYSICS')
        col = box.column(align=True)
        col.prop(obj.rigid_body, "friction", slider=True)
        col.prop(obj.rigid_body, "restitution", text="Bounciness", slider=True)

        # Gravity and Simulation
        box = layout.box()
        box.label(text="Simulation:", icon='WORLD')
        col = box.column(align=True)
        col.prop(context.scene, "use_gravity", text="Gravity")
        if context.scene.use_gravity:
            col.prop(context.scene, "gravity", text="")
        
        col.separator()
        col.prop(context.scene.rigidbody_world, "time_scale", text="Sim Speed", slider=True)
        
        # Bake
        box = layout.box()
        box.operator("rigidbody.bake_to_keyframes", text="Bake to Keyframes", icon='KEY_HLT')

class RIGID_BODY_OT_toggle_animated(bpy.types.Operator):
    bl_idname = "rigid_body.toggle_animated"
    bl_label = "Toggle Animated State"
    bl_description = "Toggle the animated state of the rigid body"

    def execute(self, context):
        obj = context.active_object
        if obj and obj.rigid_body:
            obj.rigid_body.kinematic = not obj.rigid_body.kinematic
        return {'FINISHED'}

# Registration
addon_keymaps = []

def register_keymaps():
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
        kmi = km.keymap_items.new("wm.call_menu", 'U', 'PRESS')
        kmi.properties.name = "VIEW3D_MT_quick_rigid"
        addon_keymaps.append((km, kmi))

def unregister_keymaps():
    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()